package pages

import (
	"encoding/json"
	"fmt"
	"html/template"
	"net/http"
	"strings"

	"gogs.gildas.ch/gildas/movies"
	"gogs.gildas.ch/gildas/movies/omdb"
)

func Router(c *movies.Collection) http.HandlerFunc {
	listHandler := List(c)
	movieHandler := Movie(c)

	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		path := r.URL.Path
		switch {
		case path == "/":
			listHandler(w, r)
			return
		case path == "/style.css":
			http.ServeFile(w, r, "templates/style.css")
			return
		case strings.HasPrefix(path, "/tt"):
			movieHandler(w, r)
			return
		}
	})
}

func List(c *movies.Collection) http.HandlerFunc {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		t, err := template.ParseFiles("templates/list.html")
		if err != nil {
			fmt.Println(err)
			return
		}

		var errs []error

		if r.Method == "POST" {
			m, err := movies.Unmarshal([]byte(r.FormValue("omdb_json")))
			if err != nil {
				w.WriteHeader(http.StatusBadRequest)
				errs = append(errs, err)
			} else {
				c.Add(m)
			}
		}

		var omdbString string
		if imdbID := r.URL.Query().Get("imdb_id"); imdbID != "" {
			b, err := omdb.Get(imdbID)
			if err != nil {
				errs = append(errs, err)
			} else {
				omdbString = string(b)
			}
		}

		t.Execute(w, map[string]interface{}{
			"Collection": c,
			"IMDBID":     r.URL.Query().Get("imdb_id"),
			"OMDBString": omdbString,
			"Errors":     errs,
		})
	})
}

func Movie(c *movies.Collection) http.HandlerFunc {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		t, err := template.ParseFiles("templates/movie.html")
		if err != nil {
			fmt.Println(err)
			return
		}

		var errs []error

		splitted := strings.Split(r.URL.Path, "/")
		if len(splitted) < 2 {
			w.WriteHeader(http.StatusNotFound)
			return
		}

		imdbID := splitted[1]

		m, ok := c.Get(imdbID)
		if !ok {
			w.WriteHeader(http.StatusNotFound)
			return
		}

		if r.Method == "POST" {
			updated, err := movies.Unmarshal([]byte(r.FormValue("movie_json")))
			if err != nil {
				errs = append(errs, err)
			} else if imdbID != updated.IMDBID {
				errs = append(errs, fmt.Errorf("you cannot change the imdb id."))
			} else {
				m = updated
				c.Update(updated)
			}
		}

		b, err := json.MarshalIndent(m, "", "  ")
		if err != nil {
			errs = append(errs, err)
		}

		t.Execute(w, map[string]interface{}{
			"Movie":     m,
			"MovieJSON": string(b),
			"Errors":    errs,
		})
	})
}
